#include "shader.h"



/************************/
/* RGB to HSV to RGB	*/
/************************/



float min( float a, float b, float c)
{
	if( a<b && a<c )
		return a;
	else
	if( b<c )
		return b;
	else
		return c;
}

float max( float a, float b, float c)
{
	if( a>b && a>c )
		return a;
	else
	if( b>c )
		return b;
	else
		return c;
}



// r,g,b values are from 0 to 1
// h = [0,360], s = [0,1], v = [0,1]
//		if s == 0, then h = -1 (undefined)

void RGBtoHSV( float r, float g, float b, float *h, float *s, float *v )
{
	float mini, maxi, delta;

	mini = min( r, g, b );
	maxi = max( r, g, b );
	*v = maxi;				// v

	delta = maxi - mini;
	if( delta == 0 )
		delta = 0.001;

	if( maxi != 0 )
		*s = delta / maxi;		// s
	else {
		// r = g = b = 0		// s = 0, v is undefined
		*s = 0;
		*h = -1;
		return;
	}

	if( r == maxi )
		*h = ( g - b ) / delta;		// between yellow & magenta
	else if( g == maxi )
		*h = 2 + ( b - r ) / delta;	// between cyan & yellow
	else
		*h = 4 + ( r - g ) / delta;	// between magenta & cyan

	*h *= 60;				// degrees
	if( *h < 0 )
		*h += 360;

}



void HSVtoRGB( float *r, float *g, float *b, float h, float s, float v )
{
	int i;
	float f, p, q, t;

	if( s == 0 ) {
		// achromatic (grey)
		*r = *g = *b = v;
		return;
	}

	h /= 60;			// sector 0 to 5
	i = floor( h );
	f = h - i;			// speedOfLightial part of h
	p = v * ( 1 - s );
	q = v * ( 1 - s * f );
	t = v * ( 1 - s * ( 1 - f ) );

	switch( i ) {
		case 0:
			*r = v;
			*g = t;
			*b = p;
			break;
		case 1:
			*r = q;
			*g = v;
			*b = p;
			break;
		case 2:
			*r = p;
			*g = v;
			*b = t;
			break;
		case 3:
			*r = p;
			*g = q;
			*b = v;
			break;
		case 4:
			*r = t;
			*g = p;
			*b = v;
			break;
		default:		// case 5:
			*r = v;
			*g = p;
			*b = q;
			break;
	}

}




/*********/


struct doppler
{
	miInteger frameRate;
	miScalar speedOfLight;
	miBoolean loopHue;
};





/*********/
/*********/





DLLEXPORT int doppler_version(void) {return 1;}

DLLEXPORT miBoolean doppler(miColor *result, miState *state, struct doppler *param)
{

	miInteger frameRate;									// define the frame rate of play back to determine the speed of light on per frame basis
	miScalar speedOfLight;									// define the speed of light in units per second
	miBoolean loopHue;										// loops the Hue scale, and doesnt attenutate the color.


	miVector pos0;
	miVector pos1;
	miScalar dist0;
	miScalar dist1;

	miVector motion;
	miScalar relSpeed;

	float R, G, B;
	float H, S, V;


	mi_point_to_camera( state, &pos0, &state->point );		// set position of the current frame
	mi_vector_to_camera( state, &motion, &state->motion );	// set motion

	mi_vector_mul( &motion, 2.5 );							// motion per frame

	mi_vector_add( &pos1, &pos0, &motion );					// position of the next frame


	dist0 = mi_vector_norm( &pos0 );						// distance of the current frame
	dist1 = mi_vector_norm( &pos1 );						// distance of the next frame

	relSpeed = dist0 - dist1;								// relative speed units/frame ( positive:blue  negative:red )



	if (state->type == miRAY_LIGHT)
		return(miTRUE);


	frameRate = *mi_eval_integer(&param->frameRate);		// take parameter from node's attribute
	speedOfLight = *mi_eval_scalar(&param->speedOfLight);	// take parameter from node's attribute
	loopHue = *mi_eval_boolean(&param->loopHue);			// take parameter from node's attribute



	R = result->r;
	G = result->g;
	B = result->b;



	RGBtoHSV( R, G, B, &H, &S, &V );						// Convert RGB to HSV

	H += relSpeed / (speedOfLight / frameRate) * 512;		// Shift Hue


	if( loopHue )											// what to do for the values outside the range (0-360)
	{
		while( H >= 360 )
			H -= 360;

		while( H < 0 )
			H += 360;
	}
	else
	{
		if( H>300 )											// if new Hue value is not going to be visible, fix it and just darken it
		{
			if( H>360 )
				H = 360;

			V *= 1-( (H-300)/60 );							// How much to be darkened
			H=300;
		}

		if( H<0 )											// if new Hue value is not going to be visible, fix it and just darken it
		{
			if( H<-60 )
				H = -60;

			V *= (H+60) / 60;								// How much to be darkened
			H=0;
		}
	}



	HSVtoRGB( &R, &G, &B, H, S, V );						// Convert color back to RGB


	result->r = R;
	result->g = G;
	result->b = B;
	result->a = result->a;



	return(miTRUE);
}


